@@ -1,15 +1,15 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# 
+#
 # Copyright 2004-2006 Zuza Software Foundation
-# 
+#
 # This file is part of translate.
 #
 # translate is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # translate is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -42,11 +42,12 @@ from translate.misc import optrecurse
 from Pootle import __version__ as pootleversion
 from translate import __version__ as toolkitversion
 from jToolkit import __version__ as jtoolkitversion
+from Pootle import statistics
 try:
   from xml.etree import ElementTree
 except ImportError:
   from elementtree import ElementTree
-# We don't need kid in this file, but this will show quickly if it is not 
+# We don't need kid in this file, but this will show quickly if it is not
 # installed. jToolkit won't complain, so we have to stop here if we don't have kid
 import kid
 import sys
@@ -92,6 +93,9 @@ class PootleServer(users.OptionalLoginAppServer, templateserver.TemplateServer):
     if not hasattr(self.instance, "baseurl"):
       setattr(self.instance, "baseurl", "/")
       changed = True
+    if not hasattr(self.instance, "enablealtsrc"):
+      setattr(self.instance, "enablealtsrc", False)
+      changed = True
     if changed:
       self.saveprefs()
 
@@ -123,13 +127,12 @@ class PootleServer(users.OptionalLoginAppServer, templateserver.TemplateServer):
         session.setlanguage(None)
         return
     session.setlanguage(None)
-        
+
   def inittranslation(self, localedir=None, localedomains=None, defaultlanguage=None):
     """initializes live translations using the Pootle PO files"""
     self.localedomains = ['jToolkit', 'pootle']
     self.localedir = None
     self.languagelist = self.potree.getlanguagecodes('pootle')
-    self.languagenames = self.potree.getlanguages()
     self.defaultlanguage = defaultlanguage
     if self.defaultlanguage is None:
       self.defaultlanguage = getattr(self.instance, "defaultlanguage", "en")
@@ -141,8 +144,6 @@ class PootleServer(users.OptionalLoginAppServer, templateserver.TemplateServer):
         self.errorhandler.logerror("Could not initialize translation:\n%s" % str(e))
     # if no translation available, set up a blank translation
     super(PootleServer, self).inittranslation()
-    # the inherited method overwrites self.languagenames, so we have to redo it
-    self.languagenames = self.potree.getlanguages()
 
   def gettranslation(self, language):
     """returns a translation object for the given language (or default if language is None)"""
@@ -299,7 +300,7 @@ class PootleServer(users.OptionalLoginAppServer, templateserver.TemplateServer):
       if 'username' in argdict:
         session.username = argdict["username"]
         message = session.localize("Login failed")
-      return users.LoginPage(session, languagenames=self.languagenames, message=message)
+      return users.LoginPage(session, languagenames=self.potree.getlanguages("pootle"), message=message)
     elif top == "register.html":
       return self.registerpage(session, argdict)
     elif top == "activate.html":
@@ -435,6 +436,7 @@ class PootleServer(users.OptionalLoginAppServer, templateserver.TemplateServer):
           else:
             dirfilter = ""
           try:
+            argdict["translate"] = 1
             return translatepage.TranslatePage(project, session, argdict, dirfilter)
           except projects.RightsError, stoppedby:
             argdict["message"] = str(stoppedby)
@@ -481,12 +483,16 @@ class PootleServer(users.OptionalLoginAppServer, templateserver.TemplateServer):
             page.etag = str(etag)
           else:
             page = widgets.PlainContents(filepath_or_contents)
-          if extension == "po" or extension == "csv":
-            page.content_type = "text/plain; charset=UTF-8"
-          elif extension == "xlf" or extension == "ts":
-            page.content_type = "text/xml; charset=UTF-8"
+          if extension == "po":
+            page.content_type = "text/x-gettext-translation; charset=UTF-8"
+          elif extension == "csv":
+            page.content_type = "text/csv; charset=UTF-8"
+          elif extension == "xlf":
+            page.content_type = "application/x-xliff; charset=UTF-8"
+          elif extension == "ts":
+            page.content_type = "application/x-linguist; charset=UTF-8"
           elif extension == "mo":
-            page.content_type = "application/octet-stream"
+            page.content_type = "application/x-gettext-translation"
           return page
         elif bottom.endswith(".zip"):
           if not "archive" in project.getrights(session):
@@ -524,6 +530,57 @@ class PootleServer(users.OptionalLoginAppServer, templateserver.TemplateServer):
           return indexpage.ProjectIndex(project, session, argdict, os.path.join(*pathwords))
     return None
 
+  # I implemented buildpage and sendpage here so that jToolkit's template system could
+  # be circumvented. This code originally comes from jToolkit and is still close to
+  # the original code.
+
+  def buildpage(self, filename, context, loadurl=None, localize=None, innerid=None):
+    """build a response for the template with the vars in context"""
+    context = templateserver.attribify(context)
+    t = kid.Template(filename, **context)
+    try:
+      return t.serialize(output="xhtml")
+    except Exception, e:
+      tb = sys.exc_info()[2]
+      tb_point = tb
+      while tb_point.tb_next:
+          tb_point = tb_point.tb_next
+      ancestors = tb_point.tb_frame.f_locals.get("ancestors", [])
+      xml_traceback = []
+      for ancestor in ancestors:
+          if ancestor is None: continue
+          try:
+            ancestor_str = str(ancestor)
+            #ancestor_str = kid.et.tostring(ancestor)
+          except Exception, e:
+            ancestor_str = "(could not convert %s: %s)" % (str(ancestor), str(e))
+          xml_traceback.append(ancestor_str)
+      context_str = pprint.pformat(context)
+      xml_traceback_str = "  " + "\n  ".join(xml_traceback)
+      self.errorhandler.logerror("Error converting template: %s\nContext\n%s\nXML Ancestors:\n%s\n%s\n" % (e, context_str, xml_traceback_str, self.errorhandler.traceback_str()))
+      if self.webserver.options.debug:
+        import pdb
+        pdb.post_mortem(tb)
+      raise
+
+  def sendpage(self, req, thepage):
+    """bridge to widget code to allow templating to gradually replace it"""
+    if kid is not None and hasattr(thepage, "templatename") and hasattr(thepage, "templatevars"):
+      # renders using templates rather than the underlying widget class
+      kid.enable_import(path=self.templatedir)
+      #template = kid.Template(os.path.join(self.templatedir, thepage.templatename + ".html")) #self.gettemplate(thepage.templatename)
+      loadurl = getattr(thepage, "loadurl", None)
+      if loadurl is None:
+        loadurl = getattr(self, "loadurl", None)
+      pagestring = self.buildpage(os.path.join(self.templatedir, thepage.templatename + ".html"), thepage.templatevars, loadurl, req.session.localize)
+      builtpage = widgets.PlainContents(pagestring)
+      # make sure certain attributes are retained on the built page
+      for copyattr in ('content_type', 'logresponse', 'sendfile_path', 'allowcaching', 'etag'):
+        if hasattr(thepage, copyattr):
+          setattr(builtpage, copyattr, getattr(thepage, copyattr))
+      thepage = builtpage
+    return super(PootleServer, self).sendpage(req, thepage)
+
 class PootleOptionParser(simplewebserver.WebOptionParser):
   def __init__(self):
     versionstring = "%%prog %s\njToolkit %s\nTranslate Toolkit %s\nKid %s\nElementTree %s\nPython %s (on %s/%s)" % (pootleversion.ver, jtoolkitversion.ver, toolkitversion.ver, kid.__version__, ElementTree.VERSION, sys.version, sys.platform, os.name)
@@ -534,6 +591,10 @@ class PootleOptionParser(simplewebserver.WebOptionParser):
     self.add_option('', "--refreshstats", dest="action", action="store_const", const="refreshstats",
         default="runwebserver", help="refresh the stats files instead of running the webserver")
     psycomodes=["none", "full", "profile"]
+    self.add_option('', "--statsdb_file", action="store", type="string", dest="statsdb_file",
+                    default=None, help="Specifies the location of the SQLite stats db file.")
+    self.add_option('', "--profile", action="store", type="string", dest="profile",
+                    help="Perform profiling, storing the result to the supplied filename.")
     try:
       import psyco
       self.add_option('', "--psyco", dest="psyco", default=None, choices=psycomodes, metavar="MODE",
@@ -543,7 +604,7 @@ class PootleOptionParser(simplewebserver.WebOptionParser):
 
 def checkversions():
   """Checks that version dependencies are met"""
-  if not hasattr(toolkitversion, "build") or toolkitversion.build < 11000:
+  if not hasattr(toolkitversion, "build") or toolkitversion.build < 12000:
     raise RuntimeError("requires Translate Toolkit version >= 1.1.  Current installed version is: %s" % toolkitversion.ver)
 
 def usepsyco(options):
@@ -567,6 +628,43 @@ def usepsyco(options):
   import encodings
   psyco.cannotcompile(encodings.search_function)
 
+def profile_runner(server, options):
+  import cProfile
+  import Pootle.profiling.lsprofcalltree as lsprofcalltree
+
+  def write_cache_grind(profiler, file):
+    k_cache_grind = lsprofcalltree.KCacheGrind(profiler)
+    k_cache_grind.output(file)
+    file.close()
+
+  def do_profile_run(file):
+    profiler = cProfile.Profile()
+    try:
+      profiler.runcall(simplewebserver.run, server, options)
+    finally:
+      write_cache_grind(profiler, file)
+
+  try:
+    profile_file = open(options.profile, "w+")
+    do_profile_run(profile_file)
+  except IOError, _e:
+    print "Could not open profiling file %s" % (options.profile,)
+
+def get_runner(options):
+  if getattr(options, "profile", None) != None:
+    return profile_runner
+  else:
+    return simplewebserver.run
+
+def set_stats_db(server, options):
+  def get_stats():
+    if options.statsdb_file != None:
+      return options.statsdb_file
+    else:
+      return getattr(server.instance, 'stats_db', None)
+
+  statistics.STATS_DB_FILE = get_stats()
+
 def main():
   # run the web server
   checkversions()
@@ -577,9 +675,11 @@ def main():
   if options.action != "runwebserver":
     options.servertype = "dummy"
   server = parser.getserver(options)
+  set_stats_db(server, options)
   server.options = options
   if options.action == "runwebserver":
-    simplewebserver.run(server, options)
+    run = get_runner(options)
+    run(server, options)
   elif options.action == "refreshstats":
     server.refreshstats(args)
 
