@@ -1,15 +1,15 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# 
+#
 # Copyright 2004-2006 Zuza Software Foundation
-# 
+#
 # This file is part of translate.
 #
 # translate is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-# 
+#
 # translate is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -25,7 +25,9 @@ from Pootle import pagelayout
 from Pootle import projects
 from Pootle import pootlefile
 from translate.storage import po
+from translate.misc.multistring import multistring
 import difflib
+import operator
 import urllib
 
 xml_re = re.compile("&lt;.*?&gt;")
@@ -42,6 +44,17 @@ class TranslatePage(pagelayout.PootleNavPage):
     self.argdict = argdict
     self.dirfilter = dirfilter
     self.project = project
+    self.altproject = None
+    # do we have enabled alternative source language?
+    self.enablealtsrc = getattr(session.instance, "enablealtsrc", "False")
+    if self.enablealtsrc == 'True':
+      # try to get the project if the user has chosen an alternate source language
+      altsrc = session.getaltsrclanguage()
+      if altsrc != '':
+        try:
+          self.altproject = self.project.potree.getproject(altsrc, self.project.projectcode)
+        except IndexError:
+          pass
     self.matchnames = self.getmatchnames(self.project.checker)
     self.searchtext = self.argdict.get("searchtext", "")
     # TODO: fix this in jToolkit
@@ -65,7 +78,7 @@ class TranslatePage(pagelayout.PootleNavPage):
     # TODO: clean up modes to be one variable
     self.viewmode = self.argdict.get("view", 0) and "view" in self.rights
     self.reviewmode = self.argdict.get("review", 0)
-    self.translatemode = self.argdict.get("translate", 0) or self.argdict.get("searchtext", 0) and ("translate" in self.rights or "suggest" in self.rights)
+    self.translatemode = (self.argdict.get("translate", 0) or self.argdict.get("searchtext", 0)) and ("translate" in self.rights or "suggest" in self.rights)
     notice = {}
     try:
       self.finditem()
@@ -85,10 +98,10 @@ class TranslatePage(pagelayout.PootleNavPage):
       rows = self.getdisplayrows("translate")
       icon="edit"
     if self.pofilename is not None:
-      postats = self.project.getpostats(self.pofilename)
-      blank, fuzzy = postats["blank"], postats["fuzzy"]
+      postats = self.project.getpototals(self.pofilename)
+      untranslated, fuzzy = postats["total"] - postats["translated"], postats["fuzzy"]
       translated, total = postats["translated"], postats["total"]
-      mainstats = self.localize("%d/%d translated\n(%d blank, %d fuzzy)", len(translated), len(total), len(blank), len(fuzzy))
+      mainstats = self.localize("%d/%d translated\n(%d untranslated, %d fuzzy)", translated, total, untranslated, fuzzy)
       pagelinks = self.getpagelinks("?translate=1&view=1", rows)
     navbarpath_dict = self.makenavbarpath_dict(self.project, self.session, self.pofilename, dirfilter=self.dirfilter or "")
     # templatising
@@ -118,8 +131,8 @@ class TranslatePage(pagelayout.PootleNavPage):
         "translation_title": self.localize("Translation"),
         "items": items,
         "reviewmode": self.reviewmode,
-        "accept_button": self.localize("Accept"),
-        "reject_button": self.localize("Reject"),
+        "accept_title": self.localize("Accept suggestion"),
+        "reject_title": self.localize("Reject suggestion"),
         "fuzzytext": self.localize("Fuzzy"),
         # l10n: Heading above the textarea for translator comments.
         "translator_comments_title": self.localize("Translator comments"),
@@ -130,12 +143,24 @@ class TranslatePage(pagelayout.PootleNavPage):
         # optional sections, will appear if these values are replaced
         "assign": None,
         # l10n: text next to search field
-        "search": {"title": self.localize("Search")},
+        "search": {"title": self.localize("Search"),
+                   "advanced_title": self.localize("Advanced Search"),
+                   "fields": self.getsearchfields()},
         # hidden widgets
         "searchtext": self.searchtext,
         "pofilename": givenpofilename,
         # general vars
-        "session": sessionvars, "instancetitle": instancetitle}
+        "session": sessionvars,
+        "instancetitle": instancetitle,
+        # l10n: Text displayed when an AJAX petition is being made
+        "ajax_status_text": self.localize("Working..."),
+        # l10n: Text displayed in an alert box when an AJAX petition has failed
+        "ajax_error": self.localize("Error: Something went wrong.")
+        }
+
+    if self.extra_class:
+      templatevars["search"]["extra_class"] = "nodefaultsearch"
+
     if self.showassigns and "assign" in self.rights:
       templatevars["assign"] = self.getassignbox()
     pagelayout.PootleNavPage.__init__(self, templatename, templatevars, session, bannerheight=81)
@@ -200,7 +225,10 @@ class TranslatePage(pagelayout.PootleNavPage):
       self.templatevars["assigns"] = self.getassignbox()
     if self.pofilename is not None:
       if self.matchnames:
-        checknames = [matchname.replace("check-", "", 1) for matchname in self.matchnames]
+        checknames = \
+        ["<a href='http://translate.sourceforge.net/wiki/toolkit/pofilter_tests#%(checkname)s' \
+        title='%(checkname)s' target='_blank'>%(checkname)s</a>" % \
+        {"checkname": matchname.replace("check-", "", 1)} for matchname in self.matchnames]
         # TODO: put the following parameter in quotes, since it will be foreign in all target languages
         # l10n: the parameter is the name of one of the quality checks, like "fuzzy"
         self.templatevars["checking_text"] = self.localize("checking %s", ", ".join(checknames))
@@ -300,7 +328,7 @@ class TranslatePage(pagelayout.PootleNavPage):
       value = translations[item]
       self.project.suggesttranslation(self.pofilename, item, value, self.session)
       self.lastitem = item
-      
+
     for item in submits:
       if item in skips or item not in translations:
         continue
@@ -319,9 +347,13 @@ class TranslatePage(pagelayout.PootleNavPage):
         newvalues["translator_comments"] = translator_comments
 
       self.project.updatetranslation(self.pofilename, item, newvalues, self.session)
-      
+
       self.lastitem = item
-    for item, suggid in rejects:
+
+    # Make sure we have rejects list properly sorted
+    rejects.sort(key=operator.itemgetter(1))
+    # It's necessary to loop the list reversed in order to selectively remove items
+    for item, suggid in reversed(rejects):
       value = suggestions[item, suggid]
       if isinstance(value, dict) and len(value) == 1 and 0 in value:
         value = value[0]
@@ -340,7 +372,7 @@ class TranslatePage(pagelayout.PootleNavPage):
     """returns any checker filters the user has asked to match..."""
     matchnames = []
     for checkname in self.argdict:
-      if checkname in ["fuzzy", "blank", "translated", "has-suggestion"]:
+      if checkname in ["fuzzy", "untranslated", "translated"]:
         matchnames.append(checkname)
       elif checkname in checker.getfilters():
         matchnames.append("check-" + checkname)
@@ -359,7 +391,9 @@ class TranslatePage(pagelayout.PootleNavPage):
     item = self.argdict.pop("item", None)
     if item is None:
       try:
-        search = pootlefile.Search(dirfilter=self.dirfilter, matchnames=self.matchnames, searchtext=self.searchtext)
+        # Retrieve the search fields we want to search for
+        fields = [f["name"] for f in self.getsearchfields() if f["value"] == "1"]
+        search = pootlefile.Search(dirfilter=self.dirfilter, matchnames=self.matchnames, searchtext=self.searchtext, searchfields=fields)
         # TODO: find a nicer way to let people search stuff assigned to them (does it by default now)
         # search.assignedto = self.argdict.get("assignedto", self.session.username)
         search.assignedto = self.argdict.get("assignedto", None)
@@ -423,12 +457,18 @@ class TranslatePage(pagelayout.PootleNavPage):
   def maketable(self):
     self.translations = self.gettranslations()
     items = []
-    if self.reviewmode and self.item is not None:
+    if (self.reviewmode or self.translatemode) and self.item is not None:
       suggestions = {self.item: self.project.getsuggestions(self.pofilename, self.item)}
     for row, unit in enumerate(self.translations):
       tmsuggestions = []
-      orig = unit.source.strings
-      trans = unit.target.strings
+      if isinstance(unit.source, multistring):
+        orig = unit.source.strings
+      else:
+        orig = [unit.source]
+      if isinstance(unit.target, multistring):
+        trans = unit.target.strings
+      else:
+        trans = [unit.target]
       nplurals, plurals = self.project.getpofile(self.pofilename).getheaderplural()
       try:
         if len(orig) > 1:
@@ -457,11 +497,22 @@ class TranslatePage(pagelayout.PootleNavPage):
           message_context = "".join(unit.getcontext())
         tmsuggestions = self.project.gettmsuggestions(self.pofilename, self.item)
         tmsuggestions.extend(self.project.getterminology(self.session, self.pofilename, self.item))
-        
-        if self.reviewmode:
+
+        if self.translatemode or self.reviewmode:
           translator_comments = self.escapetext(unit.getnotes(origin="translator"), stripescapes=True)
-          itemsuggestions = [suggestion.target.strings for suggestion in suggestions[item]]
+          itemsuggestions = []
+          for suggestion in suggestions[item]:
+            if suggestion.hasplural():
+              itemsuggestions.append(suggestion.target.strings)
+            else:
+              itemsuggestions.append([suggestion.target])
           transmerge = self.gettransreview(item, trans, itemsuggestions)
+          transedit = self.gettransedit(item, trans)
+          # Make sure we don't overwrite the diff attribute in case it's plural
+          if len(trans) > 1:
+            for i, f in enumerate(transedit["forms"]):
+              transedit["forms"][i].update(transmerge["forms"][i])
+          transmerge.update(transedit)
         else:
           transmerge = self.gettransedit(item, trans)
       else:
@@ -479,13 +530,15 @@ class TranslatePage(pagelayout.PootleNavPage):
         focus_class = "translate-focus"
       else:
         focus_class = ""
-      
+
       state_class = ""
       fuzzy = None
       if unit.isfuzzy():
         state_class += "translate-translation-fuzzy"
         fuzzy = "checked"
 
+      hassuggestion = len(transdict.get("suggestions", {})) > 0
+
       itemdict = {
                  "itemid": item,
                  "orig": origdict,
@@ -500,7 +553,17 @@ class TranslatePage(pagelayout.PootleNavPage):
                  "locations": locations,
                  "message_context": message_context,
                  "tm": tmsuggestions,
+                 "hassuggestion": hassuggestion
                  }
+
+      altsrcdict = {"available": False}
+      # do we have enabled alternative source language?
+      if self.enablealtsrc == 'True':
+        # get alternate source project information in a dictionary
+        if item in self.editable:
+          altsrcdict = self.getaltsrcdict(origdict)
+      itemdict["altsrc"] = altsrcdict
+
       items.append(itemdict)
     return items
 
@@ -515,7 +578,7 @@ class TranslatePage(pagelayout.PootleNavPage):
     """Replace special characters &, <, >, add and handle escapes if asked."""
     text = text.replace("&", "&amp;") # Must be done first!
     text = text.replace("<", "&lt;").replace(">", "&gt;")
-    
+
     if stripescapes:
       text = text.replace("\n", '<br />')
       text = text.replace("\r", '<br />')
@@ -665,7 +728,7 @@ class TranslatePage(pagelayout.PootleNavPage):
         transdict["text"] = self.escapefortextarea(self.localize("Translation not possible because plural information for your language is not available. Please contact the site administrator."))
         textid = "trans%d" % item
         focusbox = textid
-        
+
       transdict["can_spell"] = spellcheck.can_check_lang(self.project.languagecode)
       transdict["spell_args"] = spellargs
       transdict["buttons"] = buttons
@@ -820,3 +883,19 @@ class TranslatePage(pagelayout.PootleNavPage):
       transdict["text"] = ""
     return transdict
 
+  def getaltsrcdict(self, origdict):
+    # TODO: handle plurals !!
+    altsrcdict = {"available": False}
+    if self.altproject is not None:
+      altsrcdict["languagecode"] = pagelayout.weblanguage(self.altproject.languagecode)
+      altsrcdict["languagename"] = self.altproject.potree.getlanguagename(self.altproject.languagecode)
+      altsrcdict["dir"] = pagelayout.languagedir(altsrcdict["languagecode"])
+      altsrcdict["title"] = self.session.tr_lang(altsrcdict["languagename"])
+      if not origdict["isplural"]:
+        orig = origdict["pure"][0]["value"]
+        altsrctext = self.altproject.ugettext(orig)
+        if altsrctext != orig and not self.reviewmode:
+          altsrcdict["text"] = self.escapetext(altsrctext)
+          altsrcdict["available"] = True
+    return altsrcdict
+
