@@ -23,7 +23,7 @@ from jToolkit import web
 from jToolkit.web import server
 from jToolkit import mailer
 from jToolkit import prefs
-from Pootle import pagelayout
+from Pootle import pagelayout, request_cache
 from translate.lang import data as langdata
 from translate.lang import factory
 from email.Header import Header
@@ -175,6 +175,7 @@ class UserOptions(pagelayout.PootlePage):
     pagetitle = self.localize("Options for: %s", session.username)
     templatename = "options"
     instancetitle = getattr(session.instance, "title", session.localize("Pootle Demo"))
+    enablealtsrc = getattr(session.instance, "enablealtsrc", False)
     sessionvars = {"status": session.status, "isopen": session.isopen, "issiteadmin": session.issiteadmin()}
     templatevars = {"pagetitle": pagetitle, "introtext": message,
         "detailstitle": self.localize("Personal Details"),
@@ -194,7 +195,12 @@ class UserOptions(pagelayout.PootlePage):
         "languages": self.getlanguageoptions(),
         "home_link": self.localize("Home page"),
         "submit_button": self.localize("Save changes"),
-        "session": sessionvars, "instancetitle": instancetitle}
+        "session": sessionvars,
+        "instancetitle": instancetitle,
+        "enablealtsrc": enablealtsrc}
+    if enablealtsrc == 'True':
+      templatevars["altsrclanguage_title"] = self.localize("Alternative Source Language")
+      templatevars["altsrclanguages"] = self.getaltsrcoptions()
     otheroptions = self.getotheroptions()
     templatevars.update(otheroptions)
     pagelayout.PootlePage.__init__(self, templatename, templatevars, session)
@@ -220,6 +226,20 @@ class UserOptions(pagelayout.PootlePage):
     languages.sort(lambda x,y: locale.strcoll(x["name"], y["name"]))
     return languages
 
+  def getaltsrcoptions(self):
+    """returns options for alternative source languages"""
+    useraltsrc = self.session.getaltsrclanguage()
+    languageoptions = self.potree.getlanguages()
+    altsrclanguages = []
+    for language, name in languageoptions:
+      altsrclanguages.append({"code": language, "name": self.tr_lang(name), "selected": language in useraltsrc and 'selected' or None})
+    # rewritten for compatibility with Python 2.3
+    # altsrclanguages.sort(cmp=locale.strcoll, key=lambda dict: dict["name"])
+    altsrclanguages.sort(lambda x,y: locale.strcoll(x["name"], y["name"]))
+    # l10n: 'None' is displayed as the first item in the alternative source languages list and disables the feature.
+    altsrclanguages.insert(0, {"code": '', "name": self.session.localize("None"), "selected": '' in useraltsrc and 'selected' or None})
+    return altsrclanguages
+
   def getotheroptions(self):
     uilanguage = getattr(self.session.prefs, "uilanguage", "")
     if not uilanguage:
@@ -257,6 +277,7 @@ class OptionalLoginAppServer(server.LoginAppServer):
       else:
         self.initlanguage(req, session)
       page = self.getpage(pathwords, session, argdict)
+      request_cache.reset()
     except Exception, e:
       # Because of the exception, 'session' might not be initialised. So let's
       # play extra safe
@@ -630,6 +651,9 @@ class PootleSession(web.session.LoginSession):
     setinterfacevalue("inputwidth", self.localize("Input width must be numeric"))
     setinterfacevalue("viewrows", self.localize("The number of rows displayed in view mode must be numeric"))
     setinterfacevalue("translaterows", self.localize("The number of rows displayed in translate mode must be numeric"))
+    useraltsrclanguage = argdict.get("altsrclanguage", "")
+    if isinstance(useraltsrclanguage, (str, unicode)):
+      setattr(self.prefs, "altsrclanguage", useraltsrclanguage)
     self.saveprefs()
 
   def getprojects(self):
@@ -642,6 +666,11 @@ class PootleSession(web.session.LoginSession):
     userlanguages = getattr(self.prefs, "languages", "")
     return [languagecode.strip() for languagecode in userlanguages.split(',') if languagecode.strip()]
 
+  def getaltsrclanguage(self):
+    """gets the user's alternative source language"""
+    useraltsrclanguage = getattr(self.prefs, "altsrclanguage", "")
+    return useraltsrclanguage.strip()
+
   def getrights(self):
     """gets the user's rights"""
     return getattr(self.prefs, "rights", None)
