@@ -201,6 +201,7 @@ def getdatafiles():
   pootlefiles.append(listfiles(join('Pootle', 'html')))
   pootlefiles.append(listfiles(join('Pootle', 'html', 'images')))
   pootlefiles.append(listfiles(join('Pootle', 'html', 'js')))
+  pootlefiles.append(listfiles(join('Pootle', 'html', 'js', 'jquery')))
   pootlefiles.append(listfiles(join('Pootle', 'html', 'doc')))
   pootlefiles.append(listfiles(join('Pootle', 'html', 'doc', 'en')))
   pootlefiles.append(listfiles(join('Pootle', 'templates')))
@@ -243,6 +244,9 @@ def buildmanifest_in(file, scripts):
   print >>file, "prune .svn"
   # translate toolkit is in the same source tree but distributed separately
   print >>file, "prune translate"
+  print >>file, "prune virtaal"
+  print >>file, "prune spelt"
+  print >>file, "prune corpuscatcher"
   print >>file, "include Pootle/*.prefs"
   print >>file, "graft Pootle/html"
   print >>file, "graft Pootle/templates"
@@ -287,7 +291,7 @@ class PootleDistribution(Distribution):
       options["innosetup"] = py2exeoptions.copy()
       options["innosetup"]["install_script"] = []
     baseattrs.update(attrs)
-    fix_bdist_rpm(os.path.basename(__file__))
+#    fix_bdist_rpm(os.path.basename(__file__))
     Distribution.__init__(self, baseattrs)
 
 def standardsetup(name, version, custompackages=[], customdatafiles=[]):
@@ -325,10 +329,10 @@ def dosetup(name, version, packages, datafiles, scripts, ext_modules=[]):
         license="GNU General Public License (GPL)",
         description=description,
         long_description=long_description,
-        author="David Fraser, translate.org.za",
+        author="Translate.org.za",
         author_email="translate-devel@lists.sourceforge.net",
-        url="http://translate.sourceforge.net/",
-        download_url="http://sourceforge.net/project/showfiles.php?group_id=91920&package_id=97082",
+        url="http://translate.sourceforge.net/wiki/pootle/index",
+        download_url="http://sourceforge.net/project/showfiles.php?group_id=91920&package_id=144807",
         platforms=["any"],
         classifiers=classifiers,
         packages=packages,
